// This file was generated by gir (https://github.com/gtk-rs/gir)
// from ../ev-girs
// from ../gir-files
// DO NOT EDIT

use crate::{Job};
use glib::{prelude::*,translate::*};

glib::wrapper! {
    #[doc(alias = "PpsJobFonts")]
    pub struct JobFonts(Object<ffi::PpsJobFonts, ffi::PpsJobFontsClass>) @extends Job;

    match fn {
        type_ => || ffi::pps_job_fonts_get_type(),
    }
}

impl JobFonts {
        pub const NONE: Option<&'static JobFonts> = None;
    

    #[doc(alias = "pps_job_fonts_new")]
    pub fn new(document: &impl IsA<papers_document::Document>) -> JobFonts {
        assert_initialized_main_thread!();
        unsafe {
            Job::from_glib_full(ffi::pps_job_fonts_new(document.as_ref().to_glib_none().0)).unsafe_cast()
        }
    }
}
