// Generated by gir (https://github.com/gtk-rs/gir @ eb5be4f1bafe)
// from ../../ev-girs (@ b1161fa5399d+)
// from ../../gir-files (@ 20031a537e40)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "PpsDocumentModelClass", sizeof(PpsDocumentModelClass), alignof(PpsDocumentModelClass));
    printf("%s;%zu;%zu\n", "PpsJobAnnotsClass", sizeof(PpsJobAnnotsClass), alignof(PpsJobAnnotsClass));
    printf("%s;%zu;%zu\n", "PpsJobAttachmentsClass", sizeof(PpsJobAttachmentsClass), alignof(PpsJobAttachmentsClass));
    printf("%s;%zu;%zu\n", "PpsJobClass", sizeof(PpsJobClass), alignof(PpsJobClass));
    printf("%s;%zu;%zu\n", "PpsJobExportClass", sizeof(PpsJobExportClass), alignof(PpsJobExportClass));
    printf("%s;%zu;%zu\n", "PpsJobFindClass", sizeof(PpsJobFindClass), alignof(PpsJobFindClass));
    printf("%s;%zu;%zu\n", "PpsJobFontsClass", sizeof(PpsJobFontsClass), alignof(PpsJobFontsClass));
    printf("%s;%zu;%zu\n", "PpsJobLayersClass", sizeof(PpsJobLayersClass), alignof(PpsJobLayersClass));
    printf("%s;%zu;%zu\n", "PpsJobLinksClass", sizeof(PpsJobLinksClass), alignof(PpsJobLinksClass));
    printf("%s;%zu;%zu\n", "PpsJobLoadClass", sizeof(PpsJobLoadClass), alignof(PpsJobLoadClass));
    printf("%s;%zu;%zu\n", "PpsJobPageDataClass", sizeof(PpsJobPageDataClass), alignof(PpsJobPageDataClass));
    printf("%s;%zu;%zu\n", "PpsJobPageDataFlags", sizeof(PpsJobPageDataFlags), alignof(PpsJobPageDataFlags));
    printf("%s;%zu;%zu\n", "PpsJobPrintClass", sizeof(PpsJobPrintClass), alignof(PpsJobPrintClass));
    printf("%s;%zu;%zu\n", "PpsJobPriority", sizeof(PpsJobPriority), alignof(PpsJobPriority));
    printf("%s;%zu;%zu\n", "PpsJobRenderTextureClass", sizeof(PpsJobRenderTextureClass), alignof(PpsJobRenderTextureClass));
    printf("%s;%zu;%zu\n", "PpsJobSaveClass", sizeof(PpsJobSaveClass), alignof(PpsJobSaveClass));
    printf("%s;%zu;%zu\n", "PpsJobThumbnailTextureClass", sizeof(PpsJobThumbnailTextureClass), alignof(PpsJobThumbnailTextureClass));
    printf("%s;%zu;%zu\n", "PpsPageLayout", sizeof(PpsPageLayout), alignof(PpsPageLayout));
    printf("%s;%zu;%zu\n", "PpsSizingMode", sizeof(PpsSizingMode), alignof(PpsSizingMode));
    printf("%s;%zu;%zu\n", "PpsView", sizeof(PpsView), alignof(PpsView));
    printf("%s;%zu;%zu\n", "PpsViewPresentation", sizeof(PpsViewPresentation), alignof(PpsViewPresentation));
    return 0;
}
