// Generated by gir (https://github.com/gtk-rs/gir @ eb5be4f1bafe)
// from ../../ev-girs (@ b1161fa5399d+)
// from ../../gir-files (@ 20031a537e40)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "PpsAnnotation", sizeof(PpsAnnotation), alignof(PpsAnnotation));
    printf("%s;%zu;%zu\n", "PpsAnnotationAttachmentClass", sizeof(PpsAnnotationAttachmentClass), alignof(PpsAnnotationAttachmentClass));
    printf("%s;%zu;%zu\n", "PpsAnnotationClass", sizeof(PpsAnnotationClass), alignof(PpsAnnotationClass));
    printf("%s;%zu;%zu\n", "PpsAnnotationTextClass", sizeof(PpsAnnotationTextClass), alignof(PpsAnnotationTextClass));
    printf("%s;%zu;%zu\n", "PpsAnnotationTextIcon", sizeof(PpsAnnotationTextIcon), alignof(PpsAnnotationTextIcon));
    printf("%s;%zu;%zu\n", "PpsAnnotationTextMarkupClass", sizeof(PpsAnnotationTextMarkupClass), alignof(PpsAnnotationTextMarkupClass));
    printf("%s;%zu;%zu\n", "PpsAnnotationTextMarkupType", sizeof(PpsAnnotationTextMarkupType), alignof(PpsAnnotationTextMarkupType));
    printf("%s;%zu;%zu\n", "PpsAnnotationType", sizeof(PpsAnnotationType), alignof(PpsAnnotationType));
    printf("%s;%zu;%zu\n", "PpsAnnotationsOverMarkup", sizeof(PpsAnnotationsOverMarkup), alignof(PpsAnnotationsOverMarkup));
    printf("%s;%zu;%zu\n", "PpsAnnotationsSaveMask", sizeof(PpsAnnotationsSaveMask), alignof(PpsAnnotationsSaveMask));
    printf("%s;%zu;%zu\n", "PpsAttachment", sizeof(PpsAttachment), alignof(PpsAttachment));
    printf("%s;%zu;%zu\n", "PpsAttachmentClass", sizeof(PpsAttachmentClass), alignof(PpsAttachmentClass));
    printf("%s;%zu;%zu\n", "PpsBackendPage", sizeof(PpsBackendPage), alignof(PpsBackendPage));
    printf("%s;%zu;%zu\n", "PpsBackendPageDestroyFunc", sizeof(PpsBackendPageDestroyFunc), alignof(PpsBackendPageDestroyFunc));
    printf("%s;%zu;%zu\n", "PpsCompressionType", sizeof(PpsCompressionType), alignof(PpsCompressionType));
    printf("%s;%zu;%zu\n", "PpsDocument", sizeof(PpsDocument), alignof(PpsDocument));
    printf("%s;%zu;%zu\n", "PpsDocumentAnnotationsInterface", sizeof(PpsDocumentAnnotationsInterface), alignof(PpsDocumentAnnotationsInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentAttachmentsInterface", sizeof(PpsDocumentAttachmentsInterface), alignof(PpsDocumentAttachmentsInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentBackendInfo", sizeof(PpsDocumentBackendInfo), alignof(PpsDocumentBackendInfo));
    printf("%s;%zu;%zu\n", "PpsDocumentClass", sizeof(PpsDocumentClass), alignof(PpsDocumentClass));
    printf("%s;%zu;%zu\n", "PpsDocumentContainsJS", sizeof(PpsDocumentContainsJS), alignof(PpsDocumentContainsJS));
    printf("%s;%zu;%zu\n", "PpsDocumentError", sizeof(PpsDocumentError), alignof(PpsDocumentError));
    printf("%s;%zu;%zu\n", "PpsDocumentFindInterface", sizeof(PpsDocumentFindInterface), alignof(PpsDocumentFindInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentFontsInterface", sizeof(PpsDocumentFontsInterface), alignof(PpsDocumentFontsInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentFormsInterface", sizeof(PpsDocumentFormsInterface), alignof(PpsDocumentFormsInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentImagesInterface", sizeof(PpsDocumentImagesInterface), alignof(PpsDocumentImagesInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentInfo", sizeof(PpsDocumentInfo), alignof(PpsDocumentInfo));
    printf("%s;%zu;%zu\n", "PpsDocumentInfoFields", sizeof(PpsDocumentInfoFields), alignof(PpsDocumentInfoFields));
    printf("%s;%zu;%zu\n", "PpsDocumentLayersInterface", sizeof(PpsDocumentLayersInterface), alignof(PpsDocumentLayersInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentLayout", sizeof(PpsDocumentLayout), alignof(PpsDocumentLayout));
    printf("%s;%zu;%zu\n", "PpsDocumentLicense", sizeof(PpsDocumentLicense), alignof(PpsDocumentLicense));
    printf("%s;%zu;%zu\n", "PpsDocumentLinksInterface", sizeof(PpsDocumentLinksInterface), alignof(PpsDocumentLinksInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentLoadFlags", sizeof(PpsDocumentLoadFlags), alignof(PpsDocumentLoadFlags));
    printf("%s;%zu;%zu\n", "PpsDocumentMediaInterface", sizeof(PpsDocumentMediaInterface), alignof(PpsDocumentMediaInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentMode", sizeof(PpsDocumentMode), alignof(PpsDocumentMode));
    printf("%s;%zu;%zu\n", "PpsDocumentPermissions", sizeof(PpsDocumentPermissions), alignof(PpsDocumentPermissions));
    printf("%s;%zu;%zu\n", "PpsDocumentPrintInterface", sizeof(PpsDocumentPrintInterface), alignof(PpsDocumentPrintInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentSecurityInterface", sizeof(PpsDocumentSecurityInterface), alignof(PpsDocumentSecurityInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentTextInterface", sizeof(PpsDocumentTextInterface), alignof(PpsDocumentTextInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentTransitionInterface", sizeof(PpsDocumentTransitionInterface), alignof(PpsDocumentTransitionInterface));
    printf("%s;%zu;%zu\n", "PpsDocumentUIHints", sizeof(PpsDocumentUIHints), alignof(PpsDocumentUIHints));
    printf("%s;%zu;%zu\n", "PpsFileExporterCapabilities", sizeof(PpsFileExporterCapabilities), alignof(PpsFileExporterCapabilities));
    printf("%s;%zu;%zu\n", "PpsFileExporterContext", sizeof(PpsFileExporterContext), alignof(PpsFileExporterContext));
    printf("%s;%zu;%zu\n", "PpsFileExporterFormat", sizeof(PpsFileExporterFormat), alignof(PpsFileExporterFormat));
    printf("%s;%zu;%zu\n", "PpsFileExporterInterface", sizeof(PpsFileExporterInterface), alignof(PpsFileExporterInterface));
    printf("%s;%zu;%zu\n", "PpsFindOptions", sizeof(PpsFindOptions), alignof(PpsFindOptions));
    printf("%s;%zu;%zu\n", "PpsFindRectangle", sizeof(PpsFindRectangle), alignof(PpsFindRectangle));
    printf("%s;%zu;%zu\n", "PpsFontDescription", sizeof(PpsFontDescription), alignof(PpsFontDescription));
    printf("%s;%zu;%zu\n", "PpsFontDescriptionClass", sizeof(PpsFontDescriptionClass), alignof(PpsFontDescriptionClass));
    printf("%s;%zu;%zu\n", "PpsFormField", sizeof(PpsFormField), alignof(PpsFormField));
    printf("%s;%zu;%zu\n", "PpsFormFieldButton", sizeof(PpsFormFieldButton), alignof(PpsFormFieldButton));
    printf("%s;%zu;%zu\n", "PpsFormFieldButtonClass", sizeof(PpsFormFieldButtonClass), alignof(PpsFormFieldButtonClass));
    printf("%s;%zu;%zu\n", "PpsFormFieldButtonType", sizeof(PpsFormFieldButtonType), alignof(PpsFormFieldButtonType));
    printf("%s;%zu;%zu\n", "PpsFormFieldChoiceClass", sizeof(PpsFormFieldChoiceClass), alignof(PpsFormFieldChoiceClass));
    printf("%s;%zu;%zu\n", "PpsFormFieldChoiceType", sizeof(PpsFormFieldChoiceType), alignof(PpsFormFieldChoiceType));
    printf("%s;%zu;%zu\n", "PpsFormFieldClass", sizeof(PpsFormFieldClass), alignof(PpsFormFieldClass));
    printf("%s;%zu;%zu\n", "PpsFormFieldSignature", sizeof(PpsFormFieldSignature), alignof(PpsFormFieldSignature));
    printf("%s;%zu;%zu\n", "PpsFormFieldSignatureClass", sizeof(PpsFormFieldSignatureClass), alignof(PpsFormFieldSignatureClass));
    printf("%s;%zu;%zu\n", "PpsFormFieldTextClass", sizeof(PpsFormFieldTextClass), alignof(PpsFormFieldTextClass));
    printf("%s;%zu;%zu\n", "PpsFormFieldTextType", sizeof(PpsFormFieldTextType), alignof(PpsFormFieldTextType));
    printf("%s;%zu;%zu\n", "PpsImage", sizeof(PpsImage), alignof(PpsImage));
    printf("%s;%zu;%zu\n", "PpsImageClass", sizeof(PpsImageClass), alignof(PpsImageClass));
    printf("%s;%zu;%zu\n", "PpsLayer", sizeof(PpsLayer), alignof(PpsLayer));
    printf("%s;%zu;%zu\n", "PpsLayerClass", sizeof(PpsLayerClass), alignof(PpsLayerClass));
    printf("%s;%zu;%zu\n", "PpsLinkActionClass", sizeof(PpsLinkActionClass), alignof(PpsLinkActionClass));
    printf("%s;%zu;%zu\n", "PpsLinkActionType", sizeof(PpsLinkActionType), alignof(PpsLinkActionType));
    printf("%s;%zu;%zu\n", "PpsLinkClass", sizeof(PpsLinkClass), alignof(PpsLinkClass));
    printf("%s;%zu;%zu\n", "PpsLinkDestClass", sizeof(PpsLinkDestClass), alignof(PpsLinkDestClass));
    printf("%s;%zu;%zu\n", "PpsLinkDestType", sizeof(PpsLinkDestType), alignof(PpsLinkDestType));
    printf("%s;%zu;%zu\n", "PpsMapping", sizeof(PpsMapping), alignof(PpsMapping));
    printf("%s;%zu;%zu\n", "PpsMedia", sizeof(PpsMedia), alignof(PpsMedia));
    printf("%s;%zu;%zu\n", "PpsMediaClass", sizeof(PpsMediaClass), alignof(PpsMediaClass));
    printf("%s;%zu;%zu\n", "PpsOutlines", sizeof(PpsOutlines), alignof(PpsOutlines));
    printf("%s;%zu;%zu\n", "PpsOutlinesClass", sizeof(PpsOutlinesClass), alignof(PpsOutlinesClass));
    printf("%s;%zu;%zu\n", "PpsPage", sizeof(PpsPage), alignof(PpsPage));
    printf("%s;%zu;%zu\n", "PpsPageClass", sizeof(PpsPageClass), alignof(PpsPageClass));
    printf("%s;%zu;%zu\n", "PpsPoint", sizeof(PpsPoint), alignof(PpsPoint));
    printf("%s;%zu;%zu\n", "PpsRectangle", sizeof(PpsRectangle), alignof(PpsRectangle));
    printf("%s;%zu;%zu\n", "PpsRenderContext", sizeof(PpsRenderContext), alignof(PpsRenderContext));
    printf("%s;%zu;%zu\n", "PpsRenderContextClass", sizeof(PpsRenderContextClass), alignof(PpsRenderContextClass));
    printf("%s;%zu;%zu\n", "PpsSelectionInterface", sizeof(PpsSelectionInterface), alignof(PpsSelectionInterface));
    printf("%s;%zu;%zu\n", "PpsSelectionStyle", sizeof(PpsSelectionStyle), alignof(PpsSelectionStyle));
    printf("%s;%zu;%zu\n", "PpsTransitionEffect", sizeof(PpsTransitionEffect), alignof(PpsTransitionEffect));
    printf("%s;%zu;%zu\n", "PpsTransitionEffectAlignment", sizeof(PpsTransitionEffectAlignment), alignof(PpsTransitionEffectAlignment));
    printf("%s;%zu;%zu\n", "PpsTransitionEffectClass", sizeof(PpsTransitionEffectClass), alignof(PpsTransitionEffectClass));
    printf("%s;%zu;%zu\n", "PpsTransitionEffectDirection", sizeof(PpsTransitionEffectDirection), alignof(PpsTransitionEffectDirection));
    printf("%s;%zu;%zu\n", "PpsTransitionEffectType", sizeof(PpsTransitionEffectType), alignof(PpsTransitionEffectType));
    return 0;
}
