// Generated by gir (https://github.com/gtk-rs/gir @ eb5be4f1bafe)
// from ../../ev-girs (@ b1161fa5399d+)
// from ../../gir-files (@ 20031a537e40)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "PpsApplicationClass", sizeof(PpsApplicationClass), alignof(PpsApplicationClass));
    printf("%s;%zu;%zu\n", "PpsObjectIface", sizeof(PpsObjectIface), alignof(PpsObjectIface));
    printf("%s;%zu;%zu\n", "PpsObjectManagerClient", sizeof(PpsObjectManagerClient), alignof(PpsObjectManagerClient));
    printf("%s;%zu;%zu\n", "PpsObjectManagerClientClass", sizeof(PpsObjectManagerClientClass), alignof(PpsObjectManagerClientClass));
    printf("%s;%zu;%zu\n", "PpsObjectProxy", sizeof(PpsObjectProxy), alignof(PpsObjectProxy));
    printf("%s;%zu;%zu\n", "PpsObjectProxyClass", sizeof(PpsObjectProxyClass), alignof(PpsObjectProxyClass));
    printf("%s;%zu;%zu\n", "PpsObjectSkeleton", sizeof(PpsObjectSkeleton), alignof(PpsObjectSkeleton));
    printf("%s;%zu;%zu\n", "PpsObjectSkeletonClass", sizeof(PpsObjectSkeletonClass), alignof(PpsObjectSkeletonClass));
    printf("%s;%zu;%zu\n", "PpsPapersApplicationIface", sizeof(PpsPapersApplicationIface), alignof(PpsPapersApplicationIface));
    printf("%s;%zu;%zu\n", "PpsPapersApplicationProxy", sizeof(PpsPapersApplicationProxy), alignof(PpsPapersApplicationProxy));
    printf("%s;%zu;%zu\n", "PpsPapersApplicationProxyClass", sizeof(PpsPapersApplicationProxyClass), alignof(PpsPapersApplicationProxyClass));
    printf("%s;%zu;%zu\n", "PpsPapersApplicationSkeleton", sizeof(PpsPapersApplicationSkeleton), alignof(PpsPapersApplicationSkeleton));
    printf("%s;%zu;%zu\n", "PpsPapersApplicationSkeletonClass", sizeof(PpsPapersApplicationSkeletonClass), alignof(PpsPapersApplicationSkeletonClass));
    printf("%s;%zu;%zu\n", "PpsPapersWindowIface", sizeof(PpsPapersWindowIface), alignof(PpsPapersWindowIface));
    printf("%s;%zu;%zu\n", "PpsPapersWindowProxy", sizeof(PpsPapersWindowProxy), alignof(PpsPapersWindowProxy));
    printf("%s;%zu;%zu\n", "PpsPapersWindowProxyClass", sizeof(PpsPapersWindowProxyClass), alignof(PpsPapersWindowProxyClass));
    printf("%s;%zu;%zu\n", "PpsPapersWindowSkeleton", sizeof(PpsPapersWindowSkeleton), alignof(PpsPapersWindowSkeleton));
    printf("%s;%zu;%zu\n", "PpsPapersWindowSkeletonClass", sizeof(PpsPapersWindowSkeletonClass), alignof(PpsPapersWindowSkeletonClass));
    printf("%s;%zu;%zu\n", "PpsSidebarPageInterface", sizeof(PpsSidebarPageInterface), alignof(PpsSidebarPageInterface));
    printf("%s;%zu;%zu\n", "PpsWindow", sizeof(PpsWindow), alignof(PpsWindow));
    printf("%s;%zu;%zu\n", "PpsWindowClass", sizeof(PpsWindowClass), alignof(PpsWindowClass));
    printf("%s;%zu;%zu\n", "PpsWindowRunMode", sizeof(PpsWindowRunMode), alignof(PpsWindowRunMode));
    return 0;
}
